/******************************************************************************

AUTOLIV ELECTRONIC document.

-------------------------------------

Copyright Tuerme Inc. All rights reserved.

*******************************************************************************
C-File Template Version: 
******************************************************************************/

/*
 * Explanation:
 *    Converter from MVS Object list to TUE Fusion Object list
 */
/* PRQA S 0292 ++ *//* MKS */
/*
*/
/* PRQA S 0292 -- *//* MKS */
/*!****************************************************************************

\details
   Input adapter for TUE object fusion, converts from MVS Object List to 
   TUE Fusion object list.

******************************************************************************/

/*==================[inclusions]============================================*/

#include "MVSCamConverter.h"
#include "MVSCamConverter_Cfg.h"
#include "TueObjFusn_TrackableProps.h"
#include "TueObjFusn_ObjectListProps.h"
#include "tue_prv_fusion_math.h"
#include "tue_prv_error_management.h"
#include "Fusion_SLAM_types.h"

#include "stddef.h"  
#include "assert.h" 

#ifndef   MT_STATE_MEASURED
#define MT_STATE_MEASURED  (2U)
#endif

/*==================[macros]================================================*/
/*==================[type definitions]======================================*/
/*==================[function]==============================================*/

// boolean  MVSObjListToTueObjListConverter(CONSTP2VAR(TueObjFusn_ObjectListType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObjList,
// 	CONSTP2CONST(CamObjectList, AUTOMATIC, ObjFusn_VAR_NOINIT) inputCamObjList,
// 	const uint32 u32globalTimeStamp)
// {
// 	boolean  bSuccess = TRUE;
// 	boolean  bObjectIsValid = TRUE;
// 	static uint16 u16UpdateCount = 0u;
// 	static uint32 u32CamTimeStampPrev = 0u;
// 	float32  f32CorrLatency = FLT_ZERO;
// 	uint16   u16NumConvertedObjects = 0u;

// 	if (NULL_PTR == pNewObjList)
// 	{
// 		bSuccess = FALSE;
// 	}
// 	else if (NULL_PTR == inputCamObjList)
// 	{
// 		/* Ensure the converted object list from the previous cycle is not processed again as the update counter
// 		 * and the number of objects is not resetted
// 		 */
// 		 //pNewObjList->u16NumObjects          = 0u;
// 		 //pNewObjList->u16ListUpdateCounter   = u16UpdateCount;
// 		pNewObjList->u16NumObjects = 0u;
// 		pNewObjList->u32SensorPattern = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
// 		pNewObjList->u16ListUpdateCounter = u16UpdateCount;
// 		bSuccess = FALSE;
// 	}
// 	else
// 	{
// 		if (u32CamTimeStampPrev != inputCamObjList->sSigHeader.uiTimeStamp)
// 		{
// 			u16UpdateCount = ((u16UpdateCount + 1u) % TUEOBJFUSN_OBJECTLIST_U16LISTUPDATECOUNTER_MAX);
// 			u32CamTimeStampPrev = (inputCamObjList->sSigHeader.uiTimeStamp);
// 		}
// 		else
// 		{
// 			/* MISRA */
// 		}

// 		/* TODO: Check sensor failure flags  */

// 		if (inputCamObjList->sSigHeader.eSigStatus == FALSE)
// 		{
// 			bSuccess = FALSE;
// 		}
// 		else
// 		{
// 			pNewObjList->u16ListUpdateCounter = u16UpdateCount;

// 			/* correct latency using global timestamp */
// 			if (u32globalTimeStamp >= u32CamTimeStampPrev)
// 			{
// 				f32CorrLatency = (float32)(u32globalTimeStamp - u32CamTimeStampPrev);
// 			}
// 			else
// 			{
// 				f32CorrLatency = (float32)(0xFFFFFFFF - u32CamTimeStampPrev + u32globalTimeStamp);
// 			}
// 			f32CorrLatency = MAX(0, MIN(1000000, (f32CorrLatency)));
// 			//pNewObjList->f32MeasurementLatency  = ((pOldObjList.statusFlags.latency) + f32CorrLatency) / ALVOBJLIST_MVS_LATENCY_TO_SECONDS; //convert latency from ms to s
// 			/* convert latency from ms to s*/
// 			pNewObjList->f32MeasurementLatency = (f32CorrLatency) / TUEOBJLIST_MVS_LATENCY_TO_SECONDS / TUEOBJLIST_MVS_LATENCY_TO_SECONDS + TUEOBJLIST_MVS_DEFAULT_LATENCY_SECONDS; //convert latency from us to s

// 			pNewObjList->u32SensorPattern = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;

// 			for (uint8 i = 0; i < TUE_PRV_FUSION_MAX_OBJECTS_VISION;i ++)
// 			{
// 				bObjectIsValid = validateVisionObject(&(inputCamObjList->aObject[i]));
// 				if (bObjectIsValid)
// 				{
// 					MVSObjToTrackableConverter_convertObject(&(pNewObjList->aTrackable[u16NumConvertedObjects]),
// 						&(inputCamObjList->aObject[i]));
// 					u16NumConvertedObjects++;
// 				}
// 				else
// 				{
// 					/* MISRA */
// 				}
// 			}
// 			pNewObjList->u16NumObjects = u16NumConvertedObjects;
// 		}
// 	}
// 	return bSuccess;
// }
LOCAL void  calculateVariances_MVS(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pCamObj);

LOCAL boolean  validateVisionObject(CONSTP2CONST(CamObject, AUTOMATIC, ObjFusn_VAR_NOINIT) pVisionObj)
{
	boolean bIsValid = TRUE;

	if (tue_prv_fusion_abs(pVisionObj->Attributes.percentageProbOfExist) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fDistX) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fDistY) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fVx) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fVy) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fAx) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Kinematic.fAy) < FLT_EPSILON &&
		tue_prv_fusion_abs(pVisionObj->Geometry.fWidth) < FLT_EPSILON)
	{
		bIsValid = FALSE;
	}

	if ((pVisionObj->Attributes.percentageProbOfExist) > 100)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}


	if (((pVisionObj->Kinematic.fDistY) < -50) ||
		((pVisionObj->Kinematic.fDistY) > 50))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->Kinematic.fVx) < -127) ||
		((pVisionObj->Kinematic.fVx) > 127))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	if (((pVisionObj->Geometry.fWidth) < 0) ||
		((pVisionObj->Geometry.fWidth) > 20))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	if (pVisionObj->Classification.ObjectType > 15)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (pVisionObj->Kinematic.fAx < -20 ||
		pVisionObj->Kinematic.fAx > 20)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (pVisionObj->Kinematic.fAy < -10 ||
		pVisionObj->Kinematic.fAy > 10)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	return bIsValid;
}

static const uint16 CamClassLookup[] =
{
   TUEOBJFUSN_TRACKABLE_U16CLASS_UNKNOWN,
   TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_ADULT,
   TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_ADULT,
   TUEOBJFUSN_TRACKABLE_U16CLASS_2WHEELER_BICYCLE,
   TUEOBJFUSN_TRACKABLE_U16CLASS_2WHEELER_MOTORCYCLE,
   TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR,
   TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_TRUCK,
   TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_TRAILER,
   TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_UNKNOWN,
   TUEOBJFUSN_TRACKABLE_U16CLASS_UNKNOWN
};

/**
 * @brief Converts MVS object to TUE fusion trackable object
 * 
 * MVS object is accepted in input and then converted to
 * TUE fusion trackable type.
 * 
 * @param[in] pOldObject MVS camera object list
 * @param[out] pNewObject TUE fusion trackable object list
 * 
 * @return None
*/
void MVSObjToTrackableConverter_convertObject(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObject,
											  CONSTP2CONST(CamObject, AUTOMATIC, ObjFusn_VAR_NOINIT) pOldObject)
{
	pNewObject->fRCS = 20.f;
	pNewObject->eObjMaintenanceState = MT_STATE_MEASURED;

	/* Object Information */
	pNewObject->u16ID = pOldObject->ObjectID;
	pNewObject->u16Age =  pOldObject->Attributes.ObjectLifeCycles;	//  (uint16)(Age);
//	if(pOldObject->ObjectID != 0xff)
//	printf("MVSObjToTrackableConverter :%d, %d,%d\n", pNewObject->u16ID, Age, pOldObject->Attributes.ObjectLifeCycles);

	pNewObject->au16SensorAge[0] = 0;
	pNewObject->au16SensorAge[1] = 0;



	pNewObject->u16Lifespan = TUEOBJFUSN_TRACKABLE_U16LIFESPAN_NEW;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
//#else
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_FRONTBUMP_RELATIVE;
//#endif
	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	pNewObject->u32SensorsCurr = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
	pNewObject->u32SensorsHist = 0u;
	/* Amend by liuyang 2022-02-08  start */
	pNewObject->f32ExistenceQuality = 100;	//pOldObject->Attributes.percentageProbOfExist >= 90 ? 99 : pOldObject->Attributes.percentageProbOfExist;
	/* Amend by liuyang 2022-02-08  End */

	pNewObject->f32ObstacleProbability = TUEOBJFUSN_TRACKABLE_F32OBSTACLEPROBABILITY_MAX;

	/* Position */
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->VehiclePosX);
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->VehiclePosY);
//#else
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->VehiclePosX) - TUE_PRV_VEHICLE_CAM_TO_FRONT_BUMBER;
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->VehiclePosY) - TUE_PRV_VEHICLE_LATERAL_OFFSET_CAM;
//#endif
	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Kinematic.fDistX);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 start
#ifdef ALGO_SAFETY_DISTANCE_FOR_TEST_ONLY
//	pNewObject->vecX.data[TRACKABLE_POSX] -= ALGO_EXTEND_SAFETY_DISTANCEX_FOR_TEST;
#endif
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 end
	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Kinematic.fDistY);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	/* Velocity */
	pNewObject->vecX.data[TRACKABLE_VELX] = pOldObject->Kinematic.fVx;	// - GET_EGO_RAW_DATA_PTR->Longitudinal.VeloCorr.corrVelo;
	pNewObject->vecX.data[TRACKABLE_VELY] = pOldObject->Kinematic.fVy;

	/* Acceleration */
	pNewObject->vecX.data[TRACKABLE_ACCX] = pOldObject->Kinematic.fAx;
	pNewObject->vecX.data[TRACKABLE_ACCY] = pOldObject->Kinematic.fAy;

	pNewObject->vecX.nRows = 4u;   /* PRQA S 3120 *//* fixed vector size */

	/* Object Definition */
	pNewObject->u16MotionType = TUEOBJFUSN_TRACKABLE_U16MOTIONTYPE_UNKNOWN;

	pNewObject->u16Class = CamClassLookup[pOldObject->Classification.ObjectType];
	if (pNewObject->u16Class == TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_ADULT)
	{
		// pNewObject->vecX.data[TRACKABLE_VELX] = -GET_EGO_RAW_DATA_PTR->Longitudinal.VeloCorr.corrVelo;//pOldObject->Kinematic.fVx;
		pNewObject->vecX.data[TRACKABLE_VELX] = pOldObject->Kinematic.fVx;
		pNewObject->vecX.data[TRACKABLE_VELY] = 0;
	}

	calculateVariances_MVS(pNewObject);

	/* Geometry */
	pNewObject->f32Width = pOldObject->Geometry.fWidth;
	pNewObject->f32Heading = TUEOBJFUSN_TRACKABLE_F32HEADING_DEFAULT;
	pNewObject->f32HeadingVar = TUEOBJFUSN_TRACKABLE_F32HEADINGVAR_DEFAULT;
	pNewObject->f32YawRate = TUEOBJFUSN_TRACKABLE_F32YAWRATE_DEFAULT;
	pNewObject->f32YawRateVar = TUEOBJFUSN_TRACKABLE_F32YAWRATEVAR_DEFAULT;
	pNewObject->u16RefPoint = TUEOBJFUSN_TRACKABLE_U16REFPOINT_CENTER_OF_WIDTH;

	pNewObject->u16ClassProb = (uint16)(100);
#if PROJECT_ZT_RAM_REDUCTION_SWITCH == FALSE
	pNewObject->u8IsAsilVerified = TUEOBJLIST_MVS_BITPATTERN_ASIL;
#endif
	pNewObject->u8RadarMotionTypeInput = TU_SENSOR_MOTIONG_TYPE_INPUT_UNKNOWN;
}
#if 0
#define ObjFusn_START_SEC_SLOW_CODE
//store the other car check state, set 1 when the object pass other car valid check
static sint8 ValidOtherCarIndex[TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS];
//store the other car history location data, calculate thier velocity based on thier history location
static OtherCarKalmanStru_t OtherCarLocHistory[TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS];

//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
//store the Pedestrian check state, set 1 when the object pass pedestrian valid check
static sint8 ValidPedestrianIndex[TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS];
//store the Pedestrian history location data, calculate thier velocity based on thier history location
static PedestrianKalmanStru  PedestrianLocHistory[TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS];
//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end

//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
LOCAL  void  CalculateSpeedBasedOnLocationForPedestrian(CONSTP2CONST(CamObject_t, AUTOMATIC, ObjFusn_VAR_NOINIT) inputCamObjList)
{
	//step 1: reset ValidPedestrianIndex, find validate pedestrian ID
	memset(ValidPedestrianIndex, -1, sizeof(ValidPedestrianIndex));
	uint8 ValidPedestrianCount = 0;
	for (uint8 i = 0; i < TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS && ValidPedestrianCount < MIN(TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS, inputCamObjList->sPedestrians.uOtherPedNum); i++)
	{
		//set valid pedestrian index if pedestrian data valid
		boolean bObjectIsValid = validateVisionObject_Pedestrian(&(inputCamObjList->sPedestrians.sTargetPed[i]));
		if (TRUE == bObjectIsValid)
		{
			ValidPedestrianIndex[ValidPedestrianCount] = i;
			ValidPedestrianCount++;
		}
	}
	if (ValidPedestrianCount == 0)
	{
		return;
	}
	//step 2: loop pedestrian history array, init structure which object is not been viewed in this cycle 
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; i++)
	{
		//find current id index by last cycle object id
		Boolean findFlag = False;
		for (uint8 j = 0; j < ValidPedestrianCount; j++)
		{
			if (PedestrianLocHistory[i].ID == (inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[j]].Pedestrian_ID) && (inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[j]].Pedestrian_ID) != -1)
			{
				findFlag = True;
				break;
			}
		}
		//loss object and object id valid
		if (!findFlag && PedestrianLocHistory[i].ID != -1)
		{
			PedestrianKalmanInit(&PedestrianLocHistory[i]);
		}

	}
	//step 3: loop ValidPedestrianIndex array, init or update related kalman structure
	for (uint8 i = 0; i < ValidPedestrianCount; i++)
	{
		if (inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[i]].Pedestrian_ID != -1)
		{
			Boolean findFlag = False;
			for (uint8 j = 0; j < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; j++)
			{
				if (PedestrianLocHistory[j].ID == inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[i]].Pedestrian_ID)
				{
					findFlag = True;
					PedestrianKalmanUpdate(&PedestrianLocHistory[j], &(inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[i]]), inputCamObjList->sSigHeader.uiTimeStamp);
					PedestrianLocHistory[j].uiTimeStamp = inputCamObjList->sSigHeader.uiTimeStamp;
					break;
				}
			}
			if (!findFlag)
			{
				//find place to insert a new object kalman
				for (uint8 j = 0; j < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; j++)
				{
					if (PedestrianLocHistory[j].ID == -1)
					{
						//First kalman state update
						PedestrianKalmanFirstUpdate(&PedestrianLocHistory[j], &(inputCamObjList->sPedestrians.sTargetPed[ValidPedestrianIndex[i]]), inputCamObjList->sSigHeader.uiTimeStamp);
						break;
					}
				}
			}
		}
	}
}

void PedestrianLocHistoryInit()
{
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; i++)
	{
		PedestrianKalmanInit(&PedestrianLocHistory[i]);
	}
}
//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end

LOCAL  void  CalculateSpeedBasedOnLocation (CONSTP2CONST(CamObject_t, AUTOMATIC, ObjFusn_VAR_NOINIT) inputCamObjList)
{
	//step 1: reset ValidOtherCarIndex, find validate other car ID
	memset(ValidOtherCarIndex, -1, sizeof(ValidOtherCarIndex));
	uint8 ValidOtherCarCount = 0;
	for (uint8 i = 0; i < TUE_PRV_CAMERA_OTHER_CAR_MAX_OBJECTS && ValidOtherCarCount < MIN(TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS, inputCamObjList->sCars.uOtherCarNum); i++)
	{
		//set valid car index if other car's data valid
		boolean bObjectIsValid = validateVisionObject_OtherCar(&(inputCamObjList->sCars.sOtherCar[i]));
		if (TRUE == bObjectIsValid)
		{
			ValidOtherCarIndex[ValidOtherCarCount] = i;
			ValidOtherCarCount++;
		}
	}
	if (ValidOtherCarCount == 0)
	{
		return;
	}
	//step 2: loop Other car history array, init structure which object is not been viewed in this cycle 
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; i++)
	{
		Boolean findFlag = False;
		for (uint8 j = 0; j < ValidOtherCarCount; j++)
		{
			if (OtherCarLocHistory[i].ID == (inputCamObjList->sCars.sOtherCar[ValidOtherCarIndex[j]].Target_ID) && (inputCamObjList->sCars.sOtherCar[ValidOtherCarIndex[j]].Target_ID) != -1)
			{
				findFlag = True;
				break;
			}
		}
		if (!findFlag && OtherCarLocHistory[i].ID != -1)
		{
			OtherCarKalmanInit(&OtherCarLocHistory[i]);
		}
		
	}
	//step 3: loop ValidOtherCarIndex array, init or update related kalman structure
	for (uint8 i = 0; i < ValidOtherCarCount; i++)
	{
		if (inputCamObjList->sCars.sOtherCar[ValidOtherCarIndex[i]].Target_ID != -1)
		{
			Boolean findFlag = False;
			for (uint8 j = 0; j < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; j++)
			{
				if (OtherCarLocHistory[j].ID == inputCamObjList->sCars.sOtherCar[ValidOtherCarIndex[i]].Target_ID)
				{
					findFlag = True;
					OtherCarKalmanUpdate(&OtherCarLocHistory[j], &(inputCamObjList->sCars.sOtherCar[i]), inputCamObjList->sSigHeader.uiTimeStamp);
					OtherCarLocHistory[j].uiTimeStamp = inputCamObjList->sSigHeader.uiTimeStamp;
					break;
				}
			}
			if (!findFlag)
			{
				//find place to insert a new object kalman
				for (uint8 j = 0; j < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; j++)
				{
					if (OtherCarLocHistory[j].ID == -1)
					{
						//First kalman state update
						OtherCarKalmanFirstUpdate(&OtherCarLocHistory[j], &(inputCamObjList->sCars.sOtherCar[i]), inputCamObjList->sSigHeader.uiTimeStamp);
						break;
					}
				}
			}
		}
	}
}

void OtherCarLocHistoryInit()
{
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; i++)
	{
		OtherCarKalmanInit(&OtherCarLocHistory[i]);
	}
}

LOCAL  uint16  ConvertCamObjClass_Pedestrian(const uint8 objType)
{
	/* PRQA S 3223 8 *//* static variable requires function-scope only */
	static const uint16 CamClassLookup[] =
	{
	  TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_UNKNOWN,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_UNKNOWN,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_UNKNOWN,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_UNKNOWN,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_2WHEELER_UNKNOWN,
	};
	uint16 u16class;

	if (CamObjClass_OTHER_OBJECT > objType)
	{
		u16class = CamClassLookup[objType];
	}
	else
	{
		u16class = TUEOBJFUSN_TRACKABLE_U16CLASS_UNKNOWN;
	}

	return u16class;
}


LOCAL  uint16  ConvertCamObjClass_Car(const uint8 objType)
{
   /* PRQA S 3223 8 *//* static variable requires function-scope only */
   static const uint16 CamClassLookup[] =
   {
	  TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR,
      TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR,
      TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_TRUCK,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_TRUCK,
	  TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR
   };
   uint16 u16class;

   if (CamObjClass_OTHER_OBJECT >= objType)
   {
      u16class = CamClassLookup[objType];
   }
   else
   {
      u16class = TUEOBJFUSN_TRACKABLE_U16CLASS_UNKNOWN;
   }

   return u16class;
}

/*!****************************************************************************
* \details
*     convert  MVS Object to TUE Fusion object format
* \param[in]
*     pOldObject input pointer to MVS object
* \param[out]
*     pNewObject output pointer to fusion object
* \return
*     none
* \constraints
*     <>
* \invocation_criteria
*     <>
******************************************************************************/
LOCAL void  MVSObjToTrackableConverter_convertObject_FirstCar(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObject,
	CONSTP2CONST(FristCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pOldObject, uint32 Age)
{
	pNewObject->fRCS = 20.f;
	pNewObject->eObjMaintenanceState = MT_STATE_MEASURED;

	/* Object Information */
	pNewObject->u16ID = pOldObject->TargetID;
	pNewObject->u16Age = (uint16)(Age);
	pNewObject->u16Lifespan = TUEOBJFUSN_TRACKABLE_U16LIFESPAN_NEW;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
//#else
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_FRONTBUMP_RELATIVE;
//#endif
	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	pNewObject->u32SensorsCurr = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
	pNewObject->u32SensorsHist = 0u;
	pNewObject->f32ExistenceQuality = pOldObject->VehicleConfidence * 100 >= 90 ? 99 : pOldObject->VehicleConfidence * 100;
	pNewObject->f32ObstacleProbability = TUEOBJFUSN_TRACKABLE_F32OBSTACLEPROBABILITY_MAX;

	/* Position */
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->VehiclePosX);
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->VehiclePosY);
//#else
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->VehiclePosX) - TUE_PRV_VEHICLE_CAM_TO_FRONT_BUMBER;
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->VehiclePosY) - TUE_PRV_VEHICLE_LATERAL_OFFSET_CAM;
//#endif
	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->VehiclePosX);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 start
#ifdef ALGO_SAFETY_DISTANCE_FOR_TEST_ONLY
	pNewObject->vecX.data[TRACKABLE_POSX] -= ALGO_EXTEND_SAFETY_DISTANCEX_FOR_TEST;
#endif
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 end
	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->VehiclePosY);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	/* Velocity */
	pNewObject->vecX.data[TRACKABLE_VELX] = pOldObject->VehicleVelX;
	pNewObject->vecX.data[TRACKABLE_VELY] = FLT_ZERO;

	/* Acceleration */
	pNewObject->vecX.data[TRACKABLE_ACCX] = pOldObject->VehicleAccelX;
	pNewObject->vecX.data[TRACKABLE_ACCY] = FLT_ZERO;

	pNewObject->vecX.nRows = 4u;   /* PRQA S 3120 *//* fixed vector size */

	/* Object Definition */
	pNewObject->u16MotionType = TUEOBJFUSN_TRACKABLE_U16MOTIONTYPE_UNKNOWN;
	pNewObject->u16Class = ConvertCamObjClass_Car(pOldObject->VehicleType);

	calculateVariances_MVS(pNewObject);

	/* Geometry */
	pNewObject->f32Width = pOldObject->VehicleWidth;
	pNewObject->f32Heading = TUEOBJFUSN_TRACKABLE_F32HEADING_DEFAULT;
	pNewObject->f32HeadingVar = TUEOBJFUSN_TRACKABLE_F32HEADINGVAR_DEFAULT;
	pNewObject->f32YawRate = TUEOBJFUSN_TRACKABLE_F32YAWRATE_DEFAULT;
	pNewObject->f32YawRateVar = TUEOBJFUSN_TRACKABLE_F32YAWRATEVAR_DEFAULT;
	pNewObject->u16RefPoint = TUEOBJFUSN_TRACKABLE_U16REFPOINT_CENTER_OF_WIDTH;

	pNewObject->u16ClassProb = (uint16)(100);
#if PROJECT_ZT_RAM_REDUCTION_SWITCH == FALSE
	pNewObject->u8IsAsilVerified = TUEOBJLIST_MVS_BITPATTERN_ASIL;
#endif
	pNewObject->u8RadarMotionTypeInput = TU_SENSOR_MOTIONG_TYPE_INPUT_UNKNOWN;
}

LOCAL void  MVSObjToTrackableConverter_convertObject_OtherCar(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObject,
	CONSTP2CONST(OtherCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pOldObject, uint32 Age)
{
	pNewObject->fRCS = 20.f;
	pNewObject->eObjMaintenanceState = MT_STATE_MEASURED;

	/* Object Information */
	pNewObject->u16ID = pOldObject->Target_ID;
	pNewObject->u16Age = (uint16)(Age);
	pNewObject->u16Lifespan = TUEOBJFUSN_TRACKABLE_U16LIFESPAN_NEW;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
//#else
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_FRONTBUMP_RELATIVE;
//#endif
	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	pNewObject->u32SensorsCurr = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
	pNewObject->u32SensorsHist = 0u;
	pNewObject->f32ExistenceQuality = TUEOBJFUSN_MINIEYE_TRACKABLE_F32EXISTENCEQUALITY_DEFAULT;
	pNewObject->f32ObstacleProbability = TUEOBJFUSN_TRACKABLE_F32OBSTACLEPROBABILITY_MAX;

	/* Position */
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Vehicle_PosX);
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Vehicle_PosY);
//#else
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Vehicle_PosX) - TUE_PRV_VEHICLE_CAM_TO_FRONT_BUMBER;
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Vehicle_PosY) - TUE_PRV_VEHICLE_LATERAL_OFFSET_CAM;
//#endif
	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Vehicle_PosX);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 start
#ifdef ALGO_SAFETY_DISTANCE_FOR_TEST_ONLY
	pNewObject->vecX.data[TRACKABLE_POSX] -= ALGO_EXTEND_SAFETY_DISTANCEX_FOR_TEST;
#endif
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 end
	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Vehicle_PosY);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	/* Velocity */
	pNewObject->vecX.data[TRACKABLE_VELX] = FLT_ZERO;
	pNewObject->vecX.data[TRACKABLE_VELY] = FLT_ZERO;
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; i++)
	{
		if (OtherCarLocHistory[i].ID == pOldObject->Target_ID)
		{
			//add value protection for location and velocity estimated value
			pNewObject->vecX.data[TRACKABLE_POSX] = /*ABS(OtherCarLocHistory[i].vecX.data[TRACKABLE_POSX]) < MVS_CONVERTER_OBJECTS_F32POSX_MAX_CAMERA ? OtherCarLocHistory[i].vecX.data[TRACKABLE_POSX] :*/ (pOldObject->Vehicle_PosX);
			pNewObject->vecX.data[TRACKABLE_POSY] = /*ABS(OtherCarLocHistory[i].vecX.data[TRACKABLE_POSY]) < MVS_CONVERTER_OBJECTS_F32POSY_MAX_CAMERA ? OtherCarLocHistory[i].vecX.data[TRACKABLE_POSY] : */(pOldObject->Vehicle_PosY);
			pNewObject->vecX.data[TRACKABLE_VELX] = MINMAX(-20.f, 20.f, OtherCarLocHistory[i].vecX.data[TRACKABLE_VELX]);  //< MVS_CONVERTER_OBJECTS_F32RELVELX_MAX_CAMERA ? OtherCarLocHistory[i].vecX.data[TRACKABLE_VELX] : 0;
			pNewObject->vecX.data[TRACKABLE_VELY] = MINMAX(-10.f, 10.f, OtherCarLocHistory[i].vecX.data[TRACKABLE_VELY]);  //ABS(OtherCarLocHistory[i].vecX.data[TRACKABLE_VELY]) < MVS_CONVERTER_OBJECTS_F32RELVELY_MAX ? OtherCarLocHistory[i].vecX.data[TRACKABLE_VELY] : 0;
			break;
		}
	}
	
	/* Acceleration */
	pNewObject->vecX.data[TRACKABLE_ACCX] = FLT_ZERO;
	pNewObject->vecX.data[TRACKABLE_ACCY] = FLT_ZERO;

	pNewObject->vecX.nRows = 4u;   /* PRQA S 3120 *//* fixed vector size */

	/* Object Definition */
	pNewObject->u16MotionType = TUEOBJFUSN_TRACKABLE_U16MOTIONTYPE_UNKNOWN;
	pNewObject->u16Class = ConvertCamObjClass_Car(pOldObject->Vehicle_Type);

	calculateVariances_MVS(pNewObject);

	/* Geometry */
	pNewObject->f32Width = TUEOBJFUSN_TRACKABLE_F32WIDTH_DEFAULT;
	pNewObject->f32Heading = TUEOBJFUSN_TRACKABLE_F32HEADING_DEFAULT;
	pNewObject->f32HeadingVar = TUEOBJFUSN_TRACKABLE_F32HEADINGVAR_DEFAULT;
	pNewObject->f32YawRate = TUEOBJFUSN_TRACKABLE_F32YAWRATE_DEFAULT;
	pNewObject->f32YawRateVar = TUEOBJFUSN_TRACKABLE_F32YAWRATEVAR_DEFAULT;
	pNewObject->u16RefPoint = TUEOBJFUSN_TRACKABLE_U16REFPOINT_CENTER_OF_WIDTH;

	pNewObject->u16ClassProb = (uint16)(100);
#if PROJECT_ZT_RAM_REDUCTION_SWITCH == FALSE
	pNewObject->u8IsAsilVerified = TUEOBJLIST_MVS_BITPATTERN_ASIL;
#endif
	pNewObject->u8RadarMotionTypeInput = TU_SENSOR_MOTIONG_TYPE_INPUT_UNKNOWN;
}

LOCAL void  MVSObjToTrackableConverter_convertObject_Pedestrian(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObject,
	CONSTP2CONST(TargetPed_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pOldObject, uint32 Age)
{
	pNewObject->fRCS = 20.f;
	pNewObject->eObjMaintenanceState = MT_STATE_MEASURED;

	/* Object Information */
	pNewObject->u16ID = pOldObject->Pedestrian_ID;
	pNewObject->u16Age = (uint16)(Age);
	pNewObject->u16Lifespan = TUEOBJFUSN_TRACKABLE_U16LIFESPAN_NEW;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
//#else
//	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_FRONTBUMP_RELATIVE;
//#endif
	pNewObject->u8CoordSystem = TUEOBJFUSN_U8COORDSYSTEM_REARAXLE_RELATIVE;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	pNewObject->u32SensorsCurr = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
	pNewObject->u32SensorsHist = 0u;
	pNewObject->f32ExistenceQuality = 99.0f;
	pNewObject->f32ObstacleProbability = TUEOBJFUSN_TRACKABLE_F32OBSTACLEPROBABILITY_MAX;

	/* Position */
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 start
//#ifdef FUSION_SIMULATION_VS_CONF
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Pedestrian_PosX);
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Pedestrian_PosY);
//#else
//	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Pedestrian_PosX) - TUE_PRV_VEHICLE_CAM_TO_FRONT_BUMBER;
//	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Pedestrian_PosY) - TUE_PRV_VEHICLE_LATERAL_OFFSET_CAM;
//#endif
	pNewObject->vecX.data[TRACKABLE_POSX] = (pOldObject->Pedestrian_PosX);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 start
#ifdef ALGO_SAFETY_DISTANCE_FOR_TEST_ONLY
	pNewObject->vecX.data[TRACKABLE_POSX] -= ALGO_EXTEND_SAFETY_DISTANCEX_FOR_TEST;
#endif
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/13 added by guotao 20200428 end
	pNewObject->vecX.data[TRACKABLE_POSY] = (pOldObject->Pedestrian_PosY);
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/5 changed by guotao 20200422 end
	/* Velocity */
	pNewObject->vecX.data[TRACKABLE_VELX] = FLT_ZERO - GET_EGO_RAW_DATA_PTR->Longitudinal.VeloCorr.corrVelo;
	pNewObject->vecX.data[TRACKABLE_VELY] = FLT_ZERO;

	/* Acceleration */
	pNewObject->vecX.data[TRACKABLE_ACCX] = FLT_ZERO;
	pNewObject->vecX.data[TRACKABLE_ACCY] = FLT_ZERO;

	pNewObject->vecX.nRows = 4u;   /* PRQA S 3120 *//* fixed vector size */

	/* Object Definition */
	pNewObject->u16MotionType = TUEOBJFUSN_TRACKABLE_U16MOTIONTYPE_UNKNOWN;
	pNewObject->u16Class = ConvertCamObjClass_Pedestrian(pOldObject->Pedestrian_Type);

	calculateVariances_MVS(pNewObject);
	//camera pedestrian object PosY is more accurate than radar, use camera PosY in Fusion Process
	//changed by guotao 20200316
	pNewObject->matP.data[TRACKABLE_INDEX_VARIANCE_POSY] = FLT_ZERO;

	/* Geometry */
	pNewObject->f32Width = TUEOBJFUSN_TRACKABLE_F32WIDTH_DEFAULT / 2;
	pNewObject->f32Heading = TUEOBJFUSN_TRACKABLE_F32HEADING_DEFAULT;
	pNewObject->f32HeadingVar = TUEOBJFUSN_TRACKABLE_F32HEADINGVAR_DEFAULT;
	pNewObject->f32YawRate = TUEOBJFUSN_TRACKABLE_F32YAWRATE_DEFAULT;
	pNewObject->f32YawRateVar = TUEOBJFUSN_TRACKABLE_F32YAWRATEVAR_DEFAULT;
	pNewObject->u16RefPoint = TUEOBJFUSN_TRACKABLE_U16REFPOINT_CENTER_OF_WIDTH;

	pNewObject->u16ClassProb = (uint16)(100);

    //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
	//update kalman result
	for (uint8 i = 0; i < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; i++)
	{
		if (PedestrianLocHistory[i].ID == pOldObject->Pedestrian_ID)
		{
			pNewObject->vecX.data[TRACKABLE_POSX] = ABS(PedestrianLocHistory[i].vecX.data[TRACKABLE_POSX]) < MVS_CONVERTER_OBJECTS_F32POSX_MAX_CAMERA ? PedestrianLocHistory[i].vecX.data[TRACKABLE_POSX] : (pOldObject->Pedestrian_PosX);
			pNewObject->vecX.data[TRACKABLE_POSY] = ABS(PedestrianLocHistory[i].vecX.data[TRACKABLE_POSY]) < MVS_CONVERTER_OBJECTS_F32POSY_MAX_CAMERA ? PedestrianLocHistory[i].vecX.data[TRACKABLE_POSY] : (pOldObject->Pedestrian_PosY);
			//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/55 changed by changchang 20200622 start
			//The predicted pedstrian longitudinal speed is not used as fusion input
			//pNewObject->vecX.data[TRACKABLE_VELX] = ABS(PedestrianLocHistory[i].vecX.data[TRACKABLE_VELX]) < MVS_CONVERTER_OBJECTS_F32RELVELX_MAX_CAMERA ? PedestrianLocHistory[i].vecX.data[TRACKABLE_VELX] : 0;
			//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/55 changed by changchang 20200622 end
			pNewObject->vecX.data[TRACKABLE_VELY] = ABS(PedestrianLocHistory[i].vecX.data[TRACKABLE_VELY]) < MVS_CONVERTER_OBJECTS_F32RELVELY_MAX ? PedestrianLocHistory[i].vecX.data[TRACKABLE_VELY] : 0;
			break;
		}
	}
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end
	
#if PROJECT_ZT_RAM_REDUCTION_SWITCH == FALSE
	pNewObject->u8IsAsilVerified = TUEOBJLIST_MVS_BITPATTERN_ASIL;
#endif
	pNewObject->u8RadarMotionTypeInput = TU_SENSOR_MOTIONG_TYPE_INPUT_UNKNOWN;
}

//find min number in a float array
uint8 MinNumberOfArray(float32* InputArray, uint8 ArraySize)
{
	uint8 result = 0;
	for (int i = 1; i < ArraySize; i++)
	{
		if (InputArray[i] < InputArray[result])
		{
			result = i;
		}
	}
	return result;
}
#if 0
/*!****************************************************************************
* \details
*     convert TUE MVS object list to TUE fusion object list
* \param[in]
*     pOldObjList input pointer to MVS object list
* \param[out]
*     pNewObjList output pointer to Fusion object list
* \return
*     none
* \constraints
*     <>
* \invocation_criteria
*     <>
******************************************************************************/
/* PRQA S 1503 2 */ /**< Init method, only external function calls */
boolean  MVSObjListToTueObjListConverter(CONSTP2VAR(TueObjFusn_ObjectListType, AUTOMATIC, ObjFusn_VAR_NOINIT) pNewObjList,
	CONSTP2CONST(CamObject_t, AUTOMATIC, ObjFusn_VAR_NOINIT) inputCamObjList, const uint32 u32globalTimeStamp)
{
	uint16   u16i;
	boolean  bSuccess = TRUE;
	float32  f32CorrLatency = FLT_ZERO;
	uint16   u16NumConvertedObjects = 0u;
	boolean  bObjectIsValid = TRUE;

	/* PRQA S 3223 1 *//* static variable requires function-scope only */
	static uint16 u16UpdateCount = 0u;
	/* PRQA S 3223 1 *//* static variable requires function-scope only */
	static uint32 u32CamTimeStampPrev = 0u;

	static CamObject_t pOldObjList;
	MemCopy_MemCopy(inputCamObjList, &pOldObjList, sizeof(CamObject_t));
	
	//delete unused camera object if other car number more than 5(TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS)
	if (&pOldObjList != NULL_PTR && pOldObjList.sCars.uOtherCarNum > TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS)
	{
		float32 distanceArray[TUE_PRV_CAMERA_OTHER_CAR_MAX_OBJECTS];
		for (int i = 0; i < TUE_PRV_CAMERA_OTHER_CAR_MAX_OBJECTS; i++)
		{
			if (pOldObjList.sCars.sOtherCar[i].Vehicle_PosX == 0 && pOldObjList.sCars.sOtherCar[i].Vehicle_PosY == 0)
			{
				distanceArray[i] = 1000000;//set max distance if this object do not exist
			}
			else
			{
				distanceArray[i] = SQRT(pOldObjList.sCars.sOtherCar[i].Vehicle_PosX * pOldObjList.sCars.sOtherCar[i].Vehicle_PosX + pOldObjList.sCars.sOtherCar[i].Vehicle_PosY * pOldObjList.sCars.sOtherCar[i].Vehicle_PosY);
			}
		}

		for (int i = 0; i < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; i++)
		{
			int temp = MinNumberOfArray(distanceArray, TUE_PRV_CAMERA_OTHER_CAR_MAX_OBJECTS);
			MemCopy_MemCopy(&(inputCamObjList->sCars.sOtherCar[temp]), &(pOldObjList.sCars.sOtherCar[i]), sizeof(OtherCar_t));
			distanceArray[temp] = 1000000;//set max distance when this object has been moved to another array index
		}

		for (int i = TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; i < TUE_PRV_CAMERA_OTHER_CAR_MAX_OBJECTS; i++)
		{
			pOldObjList.sCars.sOtherCar[i].TargetNumer = 0;
			pOldObjList.sCars.sOtherCar[i].Target_ID = 0;
			pOldObjList.sCars.sOtherCar[i].Vehicle_PosX = 0;
			pOldObjList.sCars.sOtherCar[i].Vehicle_PosY = 0;
			pOldObjList.sCars.sOtherCar[i].Vehicle_Type = 0;
		}

		pOldObjList.sCars.uOtherCarNum = TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS;
	}

    //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
	//add sPedestrians number limit
	boolean bFisrtPedExistFlag  = validateVisionObject_Pedestrian(&(pOldObjList.sPedestrians.sTargetPed[0]));
	if (bFisrtPedExistFlag == TRUE)
	{
		//add first pedestrain to ped number for minieye
		pOldObjList.sPedestrians.uOtherPedNum += 1;
	}

	if (&pOldObjList != NULL_PTR && pOldObjList.sPedestrians.uOtherPedNum > TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS)
	{
		float32 distanceArrayPed[TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS];
		for (int i = 0; i < TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS; i++)
		{
			//loop all other car object and set the map distance to max
			if (pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosX == 0 && pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosY == 0)
			{
				distanceArrayPed[i] = 1000000;//set max distance if this object do not exist
			}
			else
			{
				distanceArrayPed[i] = SQRT(pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosX * pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosX + pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosY * pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosY);
			}
		}

		//If the first pedestrian exists, do not filter
		if (bFisrtPedExistFlag == TRUE)
		{
			//set min distance if the first pedestrian exist
			distanceArrayPed[0] = -1000000;
		}

		//find minmum distance object to map and set max distance for exceed max use number
		for (int i = 0; i < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; i++)
		{
			int temp = MinNumberOfArray(distanceArrayPed, TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS);
			MemCopy_MemCopy(&(inputCamObjList->sPedestrians.sTargetPed[temp]), &(pOldObjList.sPedestrians.sTargetPed[i]), sizeof(TargetPed_t));
			distanceArrayPed[temp] = 1000000;//set max distance when this object has been moved to another array index
		}

		//set exceed max number object value to default
		for (int i = TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; i < TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS; i++)
		{
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_ID = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosX = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_PosY = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_Type = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_TTC = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_VelX = 0;
			pOldObjList.sPedestrians.sTargetPed[i].Pedestrian_VelY = 0;
		}

		//set pedestrian number to max number
		pOldObjList.sPedestrians.uOtherPedNum = TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS;
	}
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end

	if (NULL_PTR == pNewObjList)
	{
		bSuccess = FALSE;
	}
	else if ((NULL_PTR == &pOldObjList) ||
		((pOldObjList.sCars.isFirstCarExist
			+ pOldObjList.sCars.uOtherCarNum
			+ pOldObjList.sPedestrians.uOtherPedNum) > TUE_PRV_FUSION_MAX_OBJECTS_VISION) /*||
		(u32CamTimeStampPrev > u32globalTimeStamp) || (u32CamTimeStampPrev > pOldObjList.statusFlags.timeStamp) */)
	{
		/* Ensure the converted object list from the previous cycle is not processed again as the update counter
		 * and the number of objects is not resetted
		 */
		 //pNewObjList->u16NumObjects          = 0u;
		 //pNewObjList->u16ListUpdateCounter   = u16UpdateCount;
		pNewObjList->u16NumObjects = 0u;
		pNewObjList->u32SensorPattern = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;
		pNewObjList->u16ListUpdateCounter = u16UpdateCount;
		bSuccess = FALSE;
	}
	else
	{
		if (u32CamTimeStampPrev != pOldObjList.sSigHeader.uiTimeStamp)
		{
			u16UpdateCount = ((u16UpdateCount + 1u) % TUEOBJFUSN_OBJECTLIST_U16LISTUPDATECOUNTER_MAX);
			u32CamTimeStampPrev = (pOldObjList.sSigHeader.uiTimeStamp);
		}
		else
		{
			/* MISRA */
		}

		/* TODO: Check sensor failure flags  */

		if (pOldObjList.sSigHeader.eSigStatus == FALSE)
		{
			bSuccess = FALSE;
		}
		else
		{
			pNewObjList->u16ListUpdateCounter = u16UpdateCount;

			/* correct latency using global timestamp */
			if (u32globalTimeStamp >= u32CamTimeStampPrev)
			{
				f32CorrLatency = (float32)(u32globalTimeStamp - u32CamTimeStampPrev);
			}
			else
			{
				f32CorrLatency = (float32)(0xFFFFFFFF - u32CamTimeStampPrev + u32globalTimeStamp);
			}
			f32CorrLatency = MAX(0, MIN(1000000, (f32CorrLatency)));
			//pNewObjList->f32MeasurementLatency  = ((pOldObjList.statusFlags.latency) + f32CorrLatency) / ALVOBJLIST_MVS_LATENCY_TO_SECONDS; //convert latency from ms to s
			/* convert latency from ms to s*/
			pNewObjList->f32MeasurementLatency = (f32CorrLatency) / TUEOBJLIST_MVS_LATENCY_TO_SECONDS / TUEOBJLIST_MVS_LATENCY_TO_SECONDS + TUEOBJLIST_MVS_DEFAULT_LATENCY_SECONDS; //convert latency from us to s

			pNewObjList->u32SensorPattern = TUEOBJFUSN_TRACKABLE_U32SENSOR_MVS_FRONT;

			if (pOldObjList.sCars.isFirstCarExist)
			{
				bObjectIsValid = validateVisionObject_FirstCar(&(pOldObjList.sCars.sFristCar));
				if (TRUE == bObjectIsValid)
				{
					MVSObjToTrackableConverter_convertObject_FirstCar(
						&(pNewObjList->aTrackable[u16NumConvertedObjects]),
						&(pOldObjList.sCars.sFristCar),
						f32CorrLatency / 1000//us -> ms
					);
					u16NumConvertedObjects++;
				}
				else
				{
					/* MISRA */
				}
			}

			//calculate other car speed based on thier history location  
			CalculateSpeedBasedOnLocation(&pOldObjList);
			for (u16i = 0u; u16i < TUE_PRV_CAMERA_USED_OTHER_CAR_MAX_OBJECTS; u16i++)
			{
				if (-1 != ValidOtherCarIndex[u16i])
				{
					MVSObjToTrackableConverter_convertObject_OtherCar(
						&(pNewObjList->aTrackable[u16NumConvertedObjects]),
						&(pOldObjList.sCars.sOtherCar[ValidOtherCarIndex[u16i]]),
						f32CorrLatency / 1000//us -> ms
					);
					/*printf("other car speed:, %d,%f,%f,%f,%f,%f\n", 
						pNewObjList->aTrackable[u16NumConvertedObjects].u16ID, 
						pNewObjList->aTrackable[u16NumConvertedObjects].vecX.data[TRACKABLE_POSX],
						pNewObjList->aTrackable[u16NumConvertedObjects].vecX.data[TRACKABLE_POSY],
						pNewObjList->aTrackable[u16NumConvertedObjects].vecX.data[TRACKABLE_VELX],
						pNewObjList->aTrackable[u16NumConvertedObjects].vecX.data[TRACKABLE_VELY],
						GET_EGO_RAW_DATA_PTR->Longitudinal.VeloCorr.corrVelo);*/
					u16NumConvertedObjects++;
				}
				else
				{
					/* MISRA */
				}
			}

            //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
			CalculateSpeedBasedOnLocationForPedestrian(&pOldObjList);
			//for (u16i = 0u; u16i < TUE_PRV_CAMERA_PEDESTRIAN_MAX_OBJECTS; u16i++)
           //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end
			for (u16i = 0u; u16i < TUE_PRV_CAMERA_USED_PEDESTRIAN_MAX_OBJECTS; u16i++)
			{
				bObjectIsValid = validateVisionObject_Pedestrian(&(pOldObjList.sPedestrians.sTargetPed[u16i]));
				if (TRUE == bObjectIsValid)
				{
					MVSObjToTrackableConverter_convertObject_Pedestrian(
						&(pNewObjList->aTrackable[u16NumConvertedObjects]),
						&(pOldObjList.sPedestrians.sTargetPed[u16i]),
						f32CorrLatency / 1000//us -> ms
					);
					u16NumConvertedObjects++;
				}
				else
				{
					/* MISRA */
				}
			}

			//set camera object rcs property according to the object class
			for (u16i = 0u; u16i < u16NumConvertedObjects; u16i++)
			{
				switch (pNewObjList->aTrackable[u16NumConvertedObjects].u16Class)
				{
				case TUEOBJFUSN_TRACKABLE_U16CLASS_UNKNOWN:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_BICYCLE;
					break;
				case TUEOBJFUSN_TRACKABLE_U16CLASS_PEDESTRIAN_UNKNOWN:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_PEDESTRIAN;
					break;
				case TUEOBJFUSN_TRACKABLE_U16CLASS_2WHEELER_UNKNOWN:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_MOTORBIKE;
					break;
				case TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_CAR:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_CAR;
					break;
				case TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_TRUCK:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_TRUCK;
					break;
				default:
					pNewObjList->aTrackable[u16NumConvertedObjects].fRCS = CAMERA_OBJECT_RCS_VALUE_TYPE_CAR;
					break;
				}
			}

			pNewObjList->u16NumObjects = u16NumConvertedObjects;
		}
	}

	return bSuccess;
}

LOCAL boolean  validateVisionObject_FirstCar(CONSTP2CONST(FristCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pVisionObj) 
{
	boolean bIsValid = TRUE;

	if (pVisionObj->VehicleConfidence == 0 &&
		pVisionObj->VehiclePosX == 0 &&
		pVisionObj->VehiclePosY == 0 &&
		pVisionObj->VehicleVelX == 0 &&
		pVisionObj->VehicleWidth == 0 &&
		pVisionObj->VehicleType == 0 &&
		pVisionObj->VehicleAccelX == 0)
	{
		bIsValid = FALSE;
	}

	if (((pVisionObj->VehicleConfidence) < MVS_CONVERTER_OBJECTS_F32VEHICLECLASSCONFIDENCE_MIN) || 
		((pVisionObj->VehicleConfidence) > MVS_CONVERTER_OBJECTS_F32VEHICLECLASSCONFIDENCE_MAX))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->VehiclePosX) < MVS_CONVERTER_OBJECTS_F32POSX_MIN_CAMERA) ||
		((pVisionObj->VehiclePosX) > MVS_CONVERTER_OBJECTS_F32POSX_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->VehiclePosY) < MVS_CONVERTER_OBJECTS_F32POSY_MIN_CAMERA) ||
		((pVisionObj->VehiclePosY) > MVS_CONVERTER_OBJECTS_F32POSY_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->VehicleVelX) < MVS_CONVERTER_OBJECTS_F32RELVELX_MIN_CAMERA) ||
		((pVisionObj->VehicleVelX) > MVS_CONVERTER_OBJECTS_F32RELVELX_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	if (((pVisionObj->VehicleWidth) < MVS_CONVERTER_OBJECTS_F32OBJWIDTH_MIN_CAMERA) ||
		((pVisionObj->VehicleWidth) > MVS_CONVERTER_OBJECTS_F32OBJWIDTH_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	if(pVisionObj->VehicleType < MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MIN_CAMERA ||
		pVisionObj->VehicleType > MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MAX_CAMERA)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (pVisionObj->VehicleAccelX < MVS_CONVERTER_OBJECTS_F32OBJACCEL_MIN_CAMERA ||
		pVisionObj->VehicleAccelX > MVS_CONVERTER_OBJECTS_F32OBJACCEL_MAX_CAMERA)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	return bIsValid;
}

LOCAL boolean  validateVisionObject_OtherCar(CONSTP2CONST(OtherCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pVisionObj)
{
	boolean bIsValid = TRUE;

	if (pVisionObj->Vehicle_PosX == 0 &&
		pVisionObj->Vehicle_PosY == 0 &&
		pVisionObj->Vehicle_Type == 0)
	{
		bIsValid = FALSE;
	}

	if (((pVisionObj->Vehicle_PosX) < MVS_CONVERTER_OBJECTS_F32POSX_MIN_CAMERA) ||
		((pVisionObj->Vehicle_PosX) > MVS_CONVERTER_OBJECTS_F32POSX_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->Vehicle_PosY) < MVS_CONVERTER_OBJECTS_F32POSY_MIN_CAMERA) ||
		((pVisionObj->Vehicle_PosY) > MVS_CONVERTER_OBJECTS_F32POSY_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (pVisionObj->Vehicle_Type < MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MIN_CAMERA ||
		pVisionObj->Vehicle_Type > MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MAX_CAMERA)
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	return bIsValid;
}
#endif
LOCAL boolean  validateVisionObject_Pedestrian(CONSTP2CONST(TargetPed_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pVisionObj)
{
	boolean bIsValid = TRUE;

	if (pVisionObj->Pedestrian_PosX == 0 &&
		pVisionObj->Pedestrian_PosY == 0 &&
		pVisionObj->Pedestrian_VelX == 0 &&
		pVisionObj->Pedestrian_VelY == 0 &&
		pVisionObj->Pedestrian_Type == 0)
	{
		bIsValid = FALSE;
	}

	if (((pVisionObj->Pedestrian_PosX) < MVS_CONVERTER_OBJECTS_F32POSX_MIN_CAMERA) ||
		((pVisionObj->Pedestrian_PosX) > MVS_CONVERTER_OBJECTS_F32POSX_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->Pedestrian_PosY) < MVS_CONVERTER_OBJECTS_F32POSY_MIN_CAMERA) ||
		((pVisionObj->Pedestrian_PosY) > MVS_CONVERTER_OBJECTS_F32POSY_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	
	if (((pVisionObj->Pedestrian_VelX) < MVS_CONVERTER_OBJECTS_F32RELVELX_MIN_CAMERA) ||
		((pVisionObj->Pedestrian_VelX) > MVS_CONVERTER_OBJECTS_F32RELVELX_MAX_CAMERA))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (((pVisionObj->Pedestrian_VelY) < MVS_CONVERTER_OBJECTS_F32RELVELY_MIN) ||
		((pVisionObj->Pedestrian_VelY) > MVS_CONVERTER_OBJECTS_F32RELVELY_MAX))
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}

	if (pVisionObj->Pedestrian_Type < MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MIN_CAMERA_PEDESTRIAN ||
		pVisionObj->Pedestrian_Type > MVS_CONVERTER_OBJECTS_U8DETECTIONTYPE_MAX_CAMERA_PEDESTRIAN )
	{
		bIsValid = FALSE;
	}
	else
	{
		/* MISRA */
	}
	return bIsValid;
}


/* ****************************************************************************

  Functionname:     OtherCarKalmanInit                                              */ /*!

  @brief            Init data for other car kalman structure. 

  @description      Including VecX, Variance P, TimeStamp, ID.

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void OtherCarKalmanInit(CONSTP2VAR(OtherCarKalmanStru_t, AUTOMATIC, ObjFusn_VAR_NOINIT) otherCarStru)
{
	otherCarStru->ID = -1;
	otherCarStru->uiTimeStamp = 0;

	memset(&(otherCarStru->matP), 0u, sizeof(stf32SymMatrix_t));
	memset(&(otherCarStru->vecX), 0u, sizeof(stf32Vec_t));
} 

/* ****************************************************************************

  Functionname:     OtherCarKalmanUpdate                                              */ /*!

  @brief            Kalman state update process

  @description      Kalman updating process based on input measurement data

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void OtherCarKalmanUpdate(CONSTP2VAR(OtherCarKalmanStru_t, AUTOMATIC, ObjFusn_VAR_NOINIT) otherCarStru, CONSTP2CONST(OtherCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) otherCarMeas, const uint32 uiTimeStamp)
{
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matEyeK;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matX_buf;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    vecKZ_hat;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matK;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) _matP;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matS;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT) _matPSym;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT) matQ;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    vecZ_hat;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matEye;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    MeasVecX;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT)    MeasVarP;
	uint16 u16nRows = otherCarStru->vecX.nRows;
	float32 f32dT_s = MAX(0, MIN(0.2, (float32)(uiTimeStamp - otherCarStru->uiTimeStamp) / 1000000));//us -> s, DelTa time calculate for kalman
	float32 f32PosVarX = FLT_ZERO;
	float32 f32PosVarY = FLT_ZERO;

	//Measurement Vector value set
	memset(&(MeasVecX), 0u, sizeof(stf32Vec_t));
	MeasVecX.nRows = 2;
	MeasVecX.data[TRACKABLE_POSX] = otherCarMeas->Vehicle_PosX;
	MeasVecX.data[TRACKABLE_POSY] = otherCarMeas->Vehicle_PosY;
	//Measurement Variance Matrix value set
	memset(&(MeasVarP), 0u, sizeof(stf32SymMatrix_t));
	MeasVarP.u16Size = 2;
	{
		f32PosVarX = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_0;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_1 * otherCarMeas->Vehicle_PosX;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_2 * otherCarMeas->Vehicle_PosY;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_3 * (otherCarMeas->Vehicle_PosX * otherCarMeas->Vehicle_PosX);
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_4 * (otherCarMeas->Vehicle_PosY * otherCarMeas->Vehicle_PosY);
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_5 * (otherCarMeas->Vehicle_PosY * otherCarMeas->Vehicle_PosX);

		f32PosVarY = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_0;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_1 * otherCarMeas->Vehicle_PosX;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_2 * otherCarMeas->Vehicle_PosY;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_3 * (otherCarMeas->Vehicle_PosX * otherCarMeas->Vehicle_PosX);
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_4 * (otherCarMeas->Vehicle_PosY * otherCarMeas->Vehicle_PosY);
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_5 * (otherCarMeas->Vehicle_PosY * otherCarMeas->Vehicle_PosX);
	}
	MeasVarP.data[TRACKABLE_INDEX_VARIANCE_POSX] = tue_prv_fusion_min_max_F32(f32PosVarX, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MAX);
	MeasVarP.data[TRACKABLE_INDEX_VARIANCE_POSY] = tue_prv_fusion_min_max_F32(f32PosVarY, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MAX);
	//Noise matrix Q value set 
	memset(&(matQ), 0u, sizeof(stf32SymMatrix_t));
	matQ.u16Size = otherCarStru->vecX.nRows;
	/* Init Q Matrix for vx, vy */
	matQ.data[TRACKABLE_INDEX_VARIANCE_POSX] = 0.2f;
	matQ.data[TRACKABLE_INDEX_VARIANCE_POSY] = 0.2f;
	matQ.data[TRACKABLE_INDEX_VARIANCE_VELX] = 0.2f;
	matQ.data[TRACKABLE_INDEX_VARIANCE_VELY] = 0.2f;

	/*   X = A * X; */
	otherCarStru->vecX.data[TRACKABLE_POSX] = otherCarStru->vecX.data[TRACKABLE_POSX] + (f32dT_s * otherCarStru->vecX.data[TRACKABLE_VELX]);
	otherCarStru->vecX.data[TRACKABLE_POSY] = otherCarStru->vecX.data[TRACKABLE_POSY] + (f32dT_s * otherCarStru->vecX.data[TRACKABLE_VELY]);
	/*   P = A * P * A' + Q; */
	LKF_PredictSymMat(&otherCarStru->matP, f32dT_s);
	f32SymMatAddSymMat(&matQ, &otherCarStru->matP, &otherCarStru->matP);

	(void)f32MatEye(&matEye, u16nRows, u16nRows);
	/** - calculate innovation z_hat */
	  /*   z_hat = z - H * x; */
	f32VecSubPart(&MeasVecX, &otherCarStru->vecX, &vecZ_hat);
	/** - calculate covariance matrix of innovation S */
	  /*   S = H * P * H' + R; */
	  /*   size of R is used and equivalent rows of P */
	f32SymMatAddSymMatPart(&MeasVarP, &otherCarStru->matP, &_matPSym); /* S = HPHT + R */
	f32SymMatToMat(&_matPSym, &matS);

	f32MatInv(&(matS)); /* S = S^-1 */
	/** - calculate Kalman gain vector K */
	  /*   K = P * H' * inv(S); */
	f32SymMatToMat(&otherCarStru->matP, &_matP);
	_matP.nCols = (MeasVarP).u16Size; /* This is equivalent to P * H' (exclude elements of P which are not in R resp S */
	f32MatMul(&_matP, &matS, &matK);
	_matP.nCols = (otherCarStru->matP).u16Size; /* reset back sicne _matP is required further done */

	/** - calculate new state vector x */
	  /*   X = X + K * z_hat; */
	f32MatMulVec(&matK, &vecZ_hat, &vecKZ_hat); /* KZ_hat = K * Z_hat */
	f32VecAdd(&otherCarStru->vecX, &vecKZ_hat, &otherCarStru->vecX); /* X = X + KZ_hat */

	/** - calculate new covariance matrix P */
	matK.nCols = otherCarStru->matP.u16Size;
	f32MatSub(&matEye, &matK, &matEyeK); /* I - K * H -> modifaction to matK */
	f32MatMul(&matEyeK, &_matP, &matX_buf); /* (I - K  * H) * P */
	f32MatToSymMat(&matX_buf, &(otherCarStru->matP));
}

/* ****************************************************************************

  Functionname:     OtherCarKalmanFirstUpdate                                              */ /*!

  @brief            Kalman structure first update for the first measurement input

  @description     

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void OtherCarKalmanFirstUpdate(CONSTP2VAR(OtherCarKalmanStru_t, AUTOMATIC, ObjFusn_VAR_NOINIT) otherCarStru, CONSTP2CONST(OtherCar_t, AUTOMATIC, ObjFusn_VAR_NOINIT) otherCarMeas, const uint32 uiTimeStamp)
{
	
	otherCarStru->ID = otherCarMeas->Target_ID;
	otherCarStru->uiTimeStamp = uiTimeStamp;

	otherCarStru->vecX.nRows = 4;
	otherCarStru->vecX.data[TRACKABLE_POSX] = otherCarMeas->Vehicle_PosX;
	otherCarStru->vecX.data[TRACKABLE_POSY] = otherCarMeas->Vehicle_PosY;
	otherCarStru->vecX.data[TRACKABLE_VELX] = 0;
	otherCarStru->vecX.data[TRACKABLE_VELY] = 0;

	otherCarStru->matP.u16Size = 4;
	otherCarStru->matP.data[TRACKABLE_INDEX_VARIANCE_POSX] = 1;
	otherCarStru->matP.data[TRACKABLE_INDEX_VARIANCE_POSY] = 1;
	otherCarStru->matP.data[TRACKABLE_INDEX_VARIANCE_VELX] = 1;
	otherCarStru->matP.data[TRACKABLE_INDEX_VARIANCE_VELY] = 1;
}


//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
/* ****************************************************************************

  Functionname:     PedestrianKalmanInit                                              */ /*!

  @brief            Init data for other car kalman structure.

  @description      Including VecX, Variance P, TimeStamp, ID.

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void PedestrianKalmanInit(CONSTP2VAR(PedestrianKalmanStru, AUTOMATIC, ObjFusn_VAR_NOINIT) pedestrianStru)
{
	pedestrianStru->ID = -1;
	pedestrianStru->uiTimeStamp = 0;

	memset(&(pedestrianStru->matP), 0u, sizeof(stf32SymMatrix_t));
	memset(&(pedestrianStru->vecX), 0u, sizeof(stf32Vec_t));
}

/* ****************************************************************************

  Functionname:     PedestrianKalmanUpdate                                              */ /*!

  @brief            Kalman state update process

  @description      Kalman updating process based on input measurement data

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void PedestrianKalmanUpdate(CONSTP2VAR(PedestrianKalmanStru, AUTOMATIC, ObjFusn_VAR_NOINIT) pedestrianStru, CONSTP2CONST(TargetPed_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pedestrianMeas, const uint32 uiTimeStamp)
{
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matEyeK;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matX_buf;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    vecKZ_hat;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matK;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) _matP;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matS;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT) _matPSym;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT) matQ;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    vecZ_hat;
	VAR(stf32Matrix_t, ObjFusn_VAR_NOINIT) matEye;
	VAR(stf32Vec_t, ObjFusn_VAR_NOINIT)    MeasVecX;
	VAR(stf32SymMatrix_t, ObjFusn_VAR_NOINIT)    MeasVarP;
	uint16 u16nRows = pedestrianStru->vecX.nRows;
	float32 f32dT_s = MAX(0, MIN(0.2, (float32)(uiTimeStamp - pedestrianStru->uiTimeStamp) / 1000000));//us -> s, DelTa time calculate for kalman
	float32 f32PosVarX = FLT_ZERO;
	float32 f32PosVarY = FLT_ZERO;

	//Measurement Vector value set
	//Set pedestrian posX,posY ,current measurement vector size 2*2 
	//Measurement vecX = [[posX],
	//	                  [posY],
	//	                       ]
	memset(&(MeasVecX), 0u, sizeof(stf32Vec_t));
	MeasVecX.nRows = 2;
	MeasVecX.data[TRACKABLE_POSX] = pedestrianMeas->Pedestrian_PosX; //update pedestrian measurement distance X
	MeasVecX.data[TRACKABLE_POSY] = pedestrianMeas->Pedestrian_PosY; //update pedestrian measurement distance Y

	//Measurement Variance Matrix value set
	memset(&(MeasVarP), 0u, sizeof(stf32SymMatrix_t)); 
	//Dynamic update measurement variance matrix R,according to object distance
	MeasVarP.u16Size = 2;
	{
		f32PosVarX = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_0;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_1 * pedestrianMeas->Pedestrian_PosX;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_2 * pedestrianMeas->Pedestrian_PosY;
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_3 * (pedestrianMeas->Pedestrian_PosX * pedestrianMeas->Pedestrian_PosX);
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_4 * (pedestrianMeas->Pedestrian_PosY * pedestrianMeas->Pedestrian_PosY);
		f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_5 * (pedestrianMeas->Pedestrian_PosY * pedestrianMeas->Pedestrian_PosX);

		f32PosVarY = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_0;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_1 * pedestrianMeas->Pedestrian_PosX;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_2 * pedestrianMeas->Pedestrian_PosY;
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_3 * (pedestrianMeas->Pedestrian_PosX * pedestrianMeas->Pedestrian_PosX);
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_4 * (pedestrianMeas->Pedestrian_PosY * pedestrianMeas->Pedestrian_PosY);
		f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_5 * (pedestrianMeas->Pedestrian_PosY * pedestrianMeas->Pedestrian_PosX);
	}
	//update measurement variance Matrix R value , matrix size 2*2
	//R = [[f32PosVarX, 0],
	//	   [0, f32PosVarY],
	//	  ]
	MeasVarP.data[TRACKABLE_INDEX_VARIANCE_POSX] = tue_prv_fusion_min_max_F32(f32PosVarX, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MAX);
	MeasVarP.data[TRACKABLE_INDEX_VARIANCE_POSY] = tue_prv_fusion_min_max_F32(f32PosVarY, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MAX);

	//Noise matrix Q value set 
	memset(&(matQ), 0u, sizeof(stf32SymMatrix_t));
	matQ.u16Size = pedestrianStru->vecX.nRows;

	// Init Q Matrix for vx, vy 
	//set Predict variance Matrix Q value , matrix size 4*4
	//Q = [[0.2,0,0,0],
	//	   [0,0.2,0,0],
	//	   [0,0,0.2,0],
	//	   [0,0,0,0.2],
	//	  ]
	matQ.data[TRACKABLE_INDEX_VARIANCE_POSX] = 0.2f;

	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/49 changed by changchang 20200612 start
	matQ.data[TRACKABLE_INDEX_VARIANCE_POSY] = 0.1f;
	//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/49 changed by changchang 20200612 end

	matQ.data[TRACKABLE_INDEX_VARIANCE_VELX] = 0.2f;
	matQ.data[TRACKABLE_INDEX_VARIANCE_VELY] = 0.2f;

	//X = A * X
	//STEP01:
	//predict new statement, This calculation is equivalent to  X = A * X
	//A = [[1,0,dt,0],
	//     [1,0,dt,0],
	//     [0,0,1, 0],
	//     [0,0,0, 1],
	//	 ]
	//vecX = [[posX],
	//        [posY],
	//        [VelX],
	//        [VelY],
	//	     ]
	pedestrianStru->vecX.data[TRACKABLE_POSX] = pedestrianStru->vecX.data[TRACKABLE_POSX] + (f32dT_s * pedestrianStru->vecX.data[TRACKABLE_VELX]);
	pedestrianStru->vecX.data[TRACKABLE_POSY] = pedestrianStru->vecX.data[TRACKABLE_POSY] + (f32dT_s * pedestrianStru->vecX.data[TRACKABLE_VELY]);
	
	//P = A * P * A' + Q;
	//STEP02:
	//predict new P matrix 
	LKF_PredictSymMat(&pedestrianStru->matP, f32dT_s);
	f32SymMatAddSymMat(&matQ, &pedestrianStru->matP, &pedestrianStru->matP);//Q + A*P*A'  = P

	//calculate innovation z_hat 
	//z_hat = z - H * x; 
	//STEP03:
	//add measument reslut to calculate z_hat for update new vecX
	f32VecSubPart(&MeasVecX, &pedestrianStru->vecX, &vecZ_hat); //z - H*x = z_hat 

	//calculate covariance matrix of innovation S
	//S = H * P * H' + R;
	//size of R is used and equivalent rows of P
	// matP is P, MeasVarP is R
	f32SymMatAddSymMatPart(&MeasVarP, &pedestrianStru->matP, &_matPSym); /* S = HPHT + R */
	f32SymMatToMat(&_matPSym, &matS);
	f32MatInv(&(matS)); /* S = S^-1 */

	//calculate Kalman gain vector K
	//K = P * H' * inv(S);
	//STEP04:
	//add measument state to calculate Kalman gain K
	f32SymMatToMat(&pedestrianStru->matP, &_matP);//
	_matP.nCols = (MeasVarP).u16Size; /* This is equivalent to P * H' (exclude elements of P which are not in R resp S */
	f32MatMul(&_matP, &matS, &matK); // K =  P * H' * inv(S)
	_matP.nCols = (pedestrianStru->matP).u16Size; /* reset back sicne _matP is required further done */

	//calculate new state vector x
	//X = X + K * z_hat;
	//STEP05:
	//accroding K gain to calculate new X(vecX)
	f32MatMulVec(&matK, &vecZ_hat, &vecKZ_hat); /* KZ_hat = K * Z_hat */
	f32VecAdd(&pedestrianStru->vecX, &vecKZ_hat, &pedestrianStru->vecX); /* X = X + KZ_hat */

	//calculate new covariance matrix P
	//STEP06:
	//accroding K gain to calculate new P(matP)
	//create the unit matrix of the same size
	(void)f32MatEye(&matEye, u16nRows, u16nRows);
	matK.nCols = pedestrianStru->matP.u16Size;
	f32MatSub(&matEye, &matK, &matEyeK); /* I - K * H -> modifaction to matK */
	f32MatMul(&matEyeK, &_matP, &matX_buf); /* (I - K  * H) * P */
	f32MatToSymMat(&matX_buf, &(pedestrianStru->matP));
}

/* ****************************************************************************

  Functionname:     PedestrianKalmanFirstUpdate                                              */ /*!

  @brief            Kalman structure first update for the first measurement input

  @description

  @param            void

  @return           void

  @pre              None
  @post             No changes


**************************************************************************** */
LOCAL void PedestrianKalmanFirstUpdate(CONSTP2VAR(PedestrianKalmanStru, AUTOMATIC, ObjFusn_VAR_NOINIT) pedestrianStru, CONSTP2CONST(TargetPed_t, AUTOMATIC, ObjFusn_VAR_NOINIT) pedestrianMeas, const uint32 uiTimeStamp)
{

	pedestrianStru->ID = pedestrianMeas->Pedestrian_ID;
	pedestrianStru->uiTimeStamp = uiTimeStamp;

	//init predict vector Xminus
	pedestrianStru->vecX.nRows = 4;
	pedestrianStru->vecX.data[TRACKABLE_POSX] = pedestrianMeas->Pedestrian_PosX;
	pedestrianStru->vecX.data[TRACKABLE_POSY] = pedestrianMeas->Pedestrian_PosY;
	//pedestrianStru->vecX.data[TRACKABLE_VELX] = 0;
	//pedestrian object is a static object
	pedestrianStru->vecX.data[TRACKABLE_VELX] = GET_EGO_RAW_DATA_PTR->Longitudinal.VeloCorr.corrVelo * -1.0F;
	pedestrianStru->vecX.data[TRACKABLE_VELY] = 0;

	
	//init meaurement&predict variance matrix P
	pedestrianStru->matP.u16Size = 4;
	pedestrianStru->matP.data[TRACKABLE_INDEX_VARIANCE_POSX] = 1;
	pedestrianStru->matP.data[TRACKABLE_INDEX_VARIANCE_POSY] = 1;
	pedestrianStru->matP.data[TRACKABLE_INDEX_VARIANCE_VELX] = 1;
	pedestrianStru->matP.data[TRACKABLE_INDEX_VARIANCE_VELY] = 1;
}
//Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end

#define ObjFusn_STOP_SEC_SLOW_CODE
#endif

/**
 * @brief Calculate MVS camera estimate uncertainty (noise) matrix
 * 
 * The P matrix in Kalman filters is calculated here.
 * 
 * @param[in] pCamObj camera track for which the P matrix is calculated
*/
LOCAL void calculateVariances_MVS(CONSTP2VAR(TueObjFusn_TrackableType, AUTOMATIC, ObjFusn_VAR_NOINIT) pCamObj)
{
	float32 f32PosX = FLT_ZERO;
	float32 f32PosY = FLT_ZERO;
	float32 f32PosVarX = FLT_ZERO;
	float32 f32PosVarY = FLT_ZERO;
	float32 f32VelVarX = FLT_ZERO;
	float32 f32VelVarY = FLT_ZERO;
	float32 f32Angle = FLT_ZERO;
	float32 f32Sin = FLT_ZERO;
	float32 f32Cos = FLT_ZERO;

	if (pCamObj != NULL_PTR)
	{
		f32PosX = pCamObj->vecX.data[TRACKABLE_POSX];
		f32PosY = pCamObj->vecX.data[TRACKABLE_POSY];
		f32Angle = tue_prv_fusion_atan2(f32PosY, f32PosX);
		f32Sin = tue_prv_fusion_sin(f32Angle);
		f32Cos = tue_prv_fusion_cos(f32Angle);

		if (((pCamObj->u16Class) & TUEOBJFUSN_TRACKABLE_U16CLASS_VEHICLE_UNKNOWN) > 0u)
		{
			f32PosVarX = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_0;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_1 * f32PosX;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_2 * f32PosY;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_3 * (f32PosX * f32PosX);
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_4 * (f32PosY * f32PosY);
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_BETA_5 * (f32PosY * f32PosX);

			f32PosVarY = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_0;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_1 * f32PosX;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_2 * f32PosY;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_3 * (f32PosX * f32PosX);
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_4 * (f32PosY * f32PosY);
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_BETA_5 * (f32PosY * f32PosX);

			f32VelVarX = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_0;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_1 * f32PosX;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_2 * f32PosY;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_3 * (f32PosX * f32PosX);
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_4 * (f32PosY * f32PosY);
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_BETA_5 * (f32PosY * f32PosX);

			f32VelVarY = TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_0;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_1 * f32PosX;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_2 * f32PosY;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_3 * (f32PosX * f32PosX);
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_4 * (f32PosY * f32PosY);
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_BETA_5 * (f32PosY * f32PosX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_POSX] = tue_prv_fusion_min_max_F32(f32PosVarX,
				TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_X_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_POSY] = tue_prv_fusion_min_max_F32(f32PosVarY,
				TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_POS_Y_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_VELX] = tue_prv_fusion_min_max_F32(f32VelVarX,
				TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_X_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_VELY] = tue_prv_fusion_min_max_F32(f32VelVarY,
				TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_VEHICLE_VEL_Y_MAX);
		}
		else
		{
			f32PosVarX = TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_0;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_1 * f32PosX;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_2 * f32PosY;
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_3 * (f32PosX * f32PosX);
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_4 * (f32PosY * f32PosY);
			f32PosVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_BETA_5 * (f32PosY * f32PosX);

			f32PosVarY = TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_0;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_1 * f32PosX;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_2 * f32PosY;
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_3 * (f32PosX * f32PosX);
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_4 * (f32PosY * f32PosY);
			f32PosVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_BETA_5 * (f32PosY * f32PosX);

			f32VelVarX = TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_0;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_1 * f32PosX;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_2 * f32PosY;
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_3 * (f32PosX * f32PosX);
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_4 * (f32PosY * f32PosY);
			f32VelVarX += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_BETA_5 * (f32PosY * f32PosX);

			f32VelVarY = TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_0;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_1 * f32PosX;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_2 * f32PosY;
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_3 * (f32PosX * f32PosX);
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_4 * (f32PosY * f32PosY);
			f32VelVarY += TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_BETA_5 * (f32PosY * f32PosX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_POSX] = tue_prv_fusion_min_max_F32(f32PosVarX,
				TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_X_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_POSY] = tue_prv_fusion_min_max_F32(f32PosVarY,
				TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_POS_Y_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_VELX] = tue_prv_fusion_min_max_F32(f32VelVarX,
				TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_X_MAX);

			pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_VELY] = tue_prv_fusion_min_max_F32(f32VelVarY,
				TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_MIN, TUEOBJLIST_MVS_SENSOR_MODEL_PEDESTRIAN_VEL_Y_MAX);
		}

		pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_ACCX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_VARIANCE_ACCY] = FLT_ZERO;

		/* Covariances */
		/* set only if sensor delivers valid covariances */
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_POSY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_VELX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_VELX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_VELY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_VELY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELX_VELY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_ACCX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_ACCX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELX_ACCX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELY_ACCX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_ACCY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_ACCY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELX_ACCY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELY_ACCY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_ACCX_ACCY] = FLT_ZERO;
		pCamObj->matP.u16Size = 4u; /* PRQA S 3120 *//* fixed vector size */

		(void)f32RotateSymMat(&(pCamObj->matP), &(pCamObj->matP), f32Sin, f32Cos);

		/* Reset Covariances as negative co-variances may result in negative variances in LKF*/
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_POSY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_VELX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_VELX] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSX_VELY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_POSY_VELY] = FLT_ZERO;
		pCamObj->matP.data[TRACKABLE_INDEX_COVARIANCE_VELX_VELY] = FLT_ZERO;

	}
	else
	{
		/* MISRA */
	}

}
/******************************************************************************
End Of File
*****************************************************************************/
